﻿Object subclass: #DipoleRadiator
	instanceVariableNames: 'listeningDistance displacement turningCenterDisplacement frontSpeakerPosition rearSpeakerPosition rearDelay rearDelaySelector rearPhase dipoleCenterPosition listenerPosition direction turnIncrement soundSpeed twoPiF frequency'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dipole Radiator'!
!DipoleRadiator commentStamp: '<historical>' prior: 0!
Iam used to calculate the radiation characteristics of two speakers at frequencies where they can be assumed to be point sources. 

rearSpeakers amplitudes must be negative!!

Ivars 
displacement is the displacement of the acoustic centers.
listeneningDistance from the center point in the middle between the speakers
turningCenterDisplacement 0.0 = middle between speakers,  displacement /2 -> front speaker
direction (of front speaker) in radians
turningCenter has coordinates of 0@0
rearDelaySelector is a symbol to plug in a function which calculates the rear delay from the frequency.

I have two samples on the class side and I'm used in one of Signal plotters samples.'!


!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 17:35'!
delayFrontSpeakerToListener
	^(listenerPosition - frontSpeakerPosition) r / soundSpeed ! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 17:35'!
delayRearSpeakerToListener
	^(listenerPosition - rearSpeakerPosition) r / soundSpeed ! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 18:20'!
dipoleCenterPosition
	^dipoleCenterPosition ! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 09:57'!
direction
	^direction! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 17:56'!
direction: aFloat 
	direction := aFloat.
	self calcDipoleCenterPosition ;
		calcSpeakerPositions ! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/13/2008 18:48'!
displacement
	^displacement! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/13/2008 18:48'!
displacement: anObject
	displacement := anObject! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 17:28'!
distanceFrontSpeakerToListener
	^(listenerPosition - frontSpeakerPosition) r! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 17:29'!
distanceRearSpeakerToListener
	^(listenerPosition - rearSpeakerPosition) r! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 17:40'!
frequency
	^frequency! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 17:41'!
frequency: aFloat 
	frequency := aFloat.
	twoPiF := Float pi * 2.0 * frequency .! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 09:27'!
frontSpeakerPosition
	^frontSpeakerPosition! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 09:28'!
listenerPosition
	^listenerPosition! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 08:44'!
listeningDistance: aFloat 
	listeningDistance := aFloat.
	listenerPosition := aFloat @ 0.0.! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 11/5/2008 19:02'!
rearAmplitude: aFloat
	^frequency <= 30.0
		ifTrue:[aFloat * -0.86] 
		ifFalse: [aFloat * -0.86]! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/16/2008 22:42'!
rearDelay
	^rearDelay! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 17:59'!
rearDelay: aFloat 
	rearDelay := aFloat! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 11/5/2008 18:49'!
rearDelayFromAngle
	"rear delay contains a constant delay to which an angle in degrees is added"

	^rearDelay + (rearPhase / 360.0 / frequency)! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/16/2008 22:45'!
rearDelayFromDistance: aFloat 
	rearDelay := aFloat / soundSpeed ! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 11/5/2008 18:52'!
rearDelaySelector: aSymbol
	rearDelaySelector  := aSymbol ! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 11/5/2008 18:49'!
rearPhase
	^rearPhase ! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 11/5/2008 18:50'!
rearPhase: aFloat 
	rearPhase  := aFloat! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 09:27'!
rearSpeakerPosition
	^rearSpeakerPosition! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 19:19'!
soundSpeed
	^soundSpeed ! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/14/2008 08:49'!
turningCenterDisplacement
	^turningCenterDisplacement! !

!DipoleRadiator methodsFor: 'accessing' stamp: 'hk 10/15/2008 09:12'!
turningCenterDisplacement: aFloat 
	turningCenterDisplacement := aFloat.
	self calcDipoleCenterPosition;
		calcSpeakerPositions ! !


!DipoleRadiator methodsFor: 'geometry' stamp: 'hk 10/14/2008 08:57'!
calcDipoleCenterPosition
	| dipoleCenterR |
	dipoleCenterR := turningCenterDisplacement * -1.0.
	dipoleCenterPosition := (dipoleCenterR * direction cos) 
				@ (dipoleCenterR * direction sin)! !

!DipoleRadiator methodsFor: 'geometry' stamp: 'hk 10/14/2008 09:53'!
calcSpeakerPositions
	| rearSpeakerR frontSpeakerR |
	frontSpeakerR := displacement / 2.0 - turningCenterDisplacement.
	rearSpeakerR := (displacement - frontSpeakerR) * -1.0.
	frontSpeakerPosition := (frontSpeakerR * direction cos) 
				@ (frontSpeakerR * direction sin).
	rearSpeakerPosition := (rearSpeakerR * direction cos) 
				@ (rearSpeakerR * direction sin)! !


!DipoleRadiator methodsFor: 'initialize-release' stamp: 'hk 10/16/2008 22:43'!
initialize
	soundSpeed := 343.0.
	frequency := 30.0.
	rearDelay := 0.0.
	rearDelaySelector := #rearDelay.
	twoPiF := Float pi * 2.0 * frequency .
	direction := 0.0.
	displacement := 0.6.
	self listeningDistance: 2.0. "also sets listenerPosition"
	self turningCenterDisplacement: displacement / 2.0.
	self calcSpeakerPositions.! !


!DipoleRadiator methodsFor: 'sound level calculations' stamp: 'hk 10/16/2008 22:38'!
amplitudeAtListenerPositionLevel: aFloat
	"Wikipedia interferenz_physik: Interferenz zweier Wellen gleicher Frequenz aber unterschiedlicher Amplitude und Phase"
	|phi1 phi2 a1 a2|
	a1 := aFloat / self distanceFrontSpeakerToListener .
	a2 := (self rearAmplitude: aFloat) / self distanceRearSpeakerToListener.
	phi1 := twoPiF * self delayFrontSpeakerToListener.
	phi2 := twoPiF * (self delayRearSpeakerToListener + (self perform: rearDelaySelector)) .
	^(a1 squared + a2 squared + (2 * a1 * a2 * (phi1 - phi2) cos)) sqrt.
	"^(self signalAtListenerPositionLevel: aFloat) max. "! !

!DipoleRadiator methodsFor: 'sound level calculations' stamp: 'hk 11/4/2008 17:08'!
amplitudeDistributionLoFreq: loFreqFloat hiFreq: hiFreqFloat noBands: noBandsInteger noDirections: noDirectInteger log: trueOrFalse 
	| toneRatio results maxVal currentFrequency|
	toneRatio := ((hiFreqFloat / loFreqFloat) ln / noBandsInteger) exp.
	currentFrequency := loFreqFloat.
	results := OrderedCollection new: noBandsInteger.
	1 to: noBandsInteger
		do: 
			[:index|
			self direction: 0.0.
			results 
				add: (self 
						polarDistibutionAmplitude: 1000.0
						frequency: currentFrequency 
						noOfDirections: noDirectInteger) asFloatArray.
			currentFrequency  := currentFrequency  * toneRatio].
	trueOrFalse 
		ifTrue: 
			[maxVal := (results collect: [:each | each max]) max.
			results withIndexDo: 
					[:each :index | |tmp|
					each /= maxVal.
					tmp := each log asFloatArray.
					tmp *= 10.0.
					results at: index put: tmp.
					]].
	^results! !

!DipoleRadiator methodsFor: 'sound level calculations' stamp: 'hk 10/16/2008 12:09'!
listenerFromFrontSpeakerAtTime: aFloat  ampitude: anotherFloat
	"which momentary amplitude reaches the listener"
	^ ((aFloat - self delayFrontSpeakerToListener) * twoPiF) sin 
		* anotherFloat 
		/ self distanceFrontSpeakerToListener.! !

!DipoleRadiator methodsFor: 'sound level calculations' stamp: 'hk 10/16/2008 22:39'!
listenerFromRearSpeakerAtTime: aFloat  ampitude: anotherFloat
	"which momentary amplitude reaches the listener"
	^ ((aFloat - self delayRearSpeakerToListener - (self perform: rearDelaySelector) ) * twoPiF) sin 
		* anotherFloat 
		/ self distanceRearSpeakerToListener .! !

!DipoleRadiator methodsFor: 'sound level calculations' stamp: 'hk 11/4/2008 15:07'!
polarDistibutionAmplitude: aFloat frequency: anotherFloat noOfDirections: noDirections 
	| dirIncrement |
	dirIncrement := Float pi * 2.0 / noDirections asFloat.
	self
		frequency: anotherFloat;
		direction: 0.0 - dirIncrement.	"for collect I have to increment first"
	^(1 to: noDirections) collect: 
			[:int | 
			self direction: direction + dirIncrement.
			(self amplitudeAtListenerPositionLevel: aFloat)]! !

!DipoleRadiator methodsFor: 'sound level calculations' stamp: 'hk 10/16/2008 14:30'!
signalAtListenerPositionLevel: aFloat
	
	^(0.0 to: 0.05 by: 0.0001) collect:
		[:time | (self listenerFromFrontSpeakerAtTime: time ampitude: aFloat) +
			(self listenerFromRearSpeakerAtTime: time 
						ampitude: (self rearAmplitude: aFloat)) ]! !

!DipoleRadiator methodsFor: 'sound level calculations' stamp: 'hk 10/14/2008 19:34'!
spectrumAtListenerPositionLevel: aFloat loFreq: aFloatLo hiFreq: aFloatHi 	
	|result |
	result := OrderedCollection new: 200.
	self frequency: aFloatLo .
	[frequency > aFloatHi] whileFalse:
		[result add: (self amplitudeAtListenerPositionLevel: aFloat).
		self frequency: frequency * 1.02 ].
	^result! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

DipoleRadiator class
	instanceVariableNames: ''!

!DipoleRadiator class methodsFor: 'examples' stamp: 'hk 10/16/2008 16:25'!
demoFrontSpectumRearCancel
	"a Spectrum from 30 to 120 Hz Amplitude 1 front direction 2m
	self demoFrontSpectumRearCancel
"

	| radiator array|
	radiator := self new.
	radiator direction: 0.0.
	radiator listeningDistance: 2.0.
	radiator turningCenterDisplacement: 0.0.
	radiator rearDelay: 0.6 / radiator soundSpeed . 
	array := radiator spectrumAtListenerPositionLevel: 1.0 
						loFreq: 30.0 
						hiFreq: 120.0.
	SignalPlotter new plot: array in: (10@10 extent: 500@200).
	SignalPlotter new plot: (array collect:[:ea| ea log * 10.0]) in: (10@250 extent: 500@200)! !

!DipoleRadiator class methodsFor: 'examples' stamp: 'hk 10/15/2008 10:10'!
demoTurning
	"test and demo the turning of the system
	self demoTurning	
"

	| screen radiator frontSpeaker rearSpeaker turningCenter dipoleCenter listener offset centerDist frontDist rearDist centerLine |
	screen := RectangleMorph new.
	screen extent: 610 @ 610.
	screen center: 307 @ 307.
	offset := screen center.
	radiator := self new.
	radiator direction: Float pi / 4.0.
	frontSpeaker := EllipseMorph new.
	frontSpeaker extent: 8 @ 8.
	rearSpeaker := frontSpeaker copy.
	turningCenter := frontSpeaker copy.
	turningCenter color: Color blue darker.
	dipoleCenter := frontSpeaker copy.
	dipoleCenter color: Color green darker.
	listener := frontSpeaker copy.
	listener color: Color red.
	radiator
		listeningDistance: 300.0;
		displacement: 90.0;
		turningCenterDisplacement: 15.0;
		calcDipoleCenterPosition;
		calcSpeakerPositions.
	frontSpeaker center: radiator frontSpeakerPosition + offset.
	screen addMorph: frontSpeaker.
	rearSpeaker center: radiator rearSpeakerPosition + offset.
	screen addMorph: rearSpeaker.
	turningCenter center: offset.
	screen addMorph: turningCenter.
	dipoleCenter center: radiator dipoleCenterPosition + offset.
	screen addMorph: dipoleCenter.
	listener center: radiator listenerPosition + offset.
	screen addMorph: listener.
	centerLine := PolygonMorph 
				vertices: { (frontSpeaker center). (rearSpeaker center) }
				color: Color black
				borderWidth: 2
				borderColor: Color black.
	screen addMorph: centerLine.
	centerDist := centerLine copy.
	centerDist borderWidth: 1.
	centerDist verticesAt: 1 put: turningCenter center;
				verticesAt: 2 put:  listener center.
	screen addMorph: centerDist.
	frontDist := centerDist copy.
	frontDist borderColor: Color blue lighter lighter.
	frontDist verticesAt: 1 put: frontSpeaker center; 
				verticesAt: 2 put:  listener center.
	screen addMorph: frontDist.
	rearDist := frontDist copy.
	rearDist borderColor: Color red.
	rearDist verticesAt: 1 put: rearSpeaker center.
	screen addMorph: rearDist.
	screen openInWorld.
	(1 to: 720 by: 1) do:
		[:index | radiator direction: index degreesToRadians;
							calcDipoleCenterPosition;
							calcSpeakerPositions.
		frontSpeaker center: radiator frontSpeakerPosition + offset.
		rearSpeaker center: radiator rearSpeakerPosition + offset.
		dipoleCenter center: radiator dipoleCenterPosition + offset.
		centerLine verticesAt: 1 put: frontSpeaker center;
					verticesAt: 2 put: rearSpeaker center.
		frontDist verticesAt: 1 put: frontSpeaker center.
		rearDist verticesAt: 1 put: rearSpeaker center.
		(Delay forMilliseconds: 10) wait.
		World doOneCycleNow .
		].
	screen delete! !


ScaleMorph subclass: #LogScaleMorph
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dipole Radiator'!
!LogScaleMorph commentStamp: 'hk 9/13/2007 19:02' prior: 0!
I'm 'ScaleMorph for Logarithmic scales.
Interpretation of tics goes within one decade. majorTick is now an Array like #(1 2 5) while minorTick is interpreted as adding to 1 in the 1 to 10 decade.

See calcMajorTics and calcMinorTics for how it's done.'

Remember 0 is forbidden.

Just overwrite the relevant methods to get logarithmic scales.

!


!LogScaleMorph methodsFor: 'drawing' stamp: 'hk 8/29/2007 11:01'!
buildLabels
	| scale x1 y1 y2 x captionMorph tickMorph offset logTics|
	"majorTics is for one decade, calc all tics in range"
	logTics := self calcMajorTics: majorTick .
	majorTickLength * minorTickLength < 0 
		ifTrue: [minorTickLength := 0 - minorTickLength].
	self removeAllMorphs.
	caption ifNotNil: 
			[captionMorph := StringMorph contents: caption.
			offset := captionAbove 
				ifTrue: [majorTickLength abs + captionMorph height + 7]
				ifFalse: [2].
			captionMorph align: captionMorph bounds bottomCenter
				with: self bounds bottomCenter - (0 @ offset).
			self addMorph: captionMorph].
	tickPrintBlock ifNotNil: 
			["Calculate the offset for the labels, depending on whether or not 
			  1) there's a caption   
			below, 2) the labels are above or below the ticks, and 3) the   
			ticks go up or down"

			offset := labelsAbove 
						ifTrue: [majorTickLength abs + minorTickLength abs + 2]
						ifFalse: [2].
			caption 
				ifNotNil: [captionAbove ifFalse: [offset := offset + captionMorph height + 2]].
			scale := (self innerBounds width - 1) / (stop log - start log) asFloat.
			x1 := self innerBounds left.
			y1 := self innerBounds bottom.
			y2 := y1 - offset.
			logTics 
				do: 
					[:v | 
					x := x1 + (scale * (v log - start log)).
					tickMorph := StringMorph contents: (tickPrintBlock value: v).
					tickMorph align: tickMorph bounds bottomCenter with: x @ y2.
					tickMorph left < self left 
						ifTrue: [tickMorph position: self left @ tickMorph top].
					tickMorph right > self right 
						ifTrue: [tickMorph position: (self right - tickMorph width) @ tickMorph top].
					self addMorph: tickMorph]]! !

!LogScaleMorph methodsFor: 'drawing' stamp: 'hk 8/29/2007 13:30'!
calcMajorTics: ticsForOneDecade 
	"ticList contains the values inside a decade"

	| frequencyList decade |
	decade := ticsForOneDecade copy.
	[start < decade last] whileTrue: [decade := decade / 10].
	[start > decade last] whileTrue: [decade := decade * 10].
	frequencyList := OrderedCollection new.
	decade do: [:freq | freq >= start ifTrue: [frequencyList add: freq]].
	[stop >= (decade last)] whileTrue: 
			[decade := decade * 10.
			decade do: [:freq | freq <= stop ifTrue: [frequencyList add: freq]]].
	^frequencyList! !

!LogScaleMorph methodsFor: 'drawing' stamp: 'hk 8/29/2007 13:30'!
calcMinorTics
	"ticList contains the values inside a decade"

	| frequencyList decade |
	decade := (1 to: 10 by: minorTick)  asArray.
	[start < decade last] whileTrue: [decade := decade / 10].
	[start > decade last] whileTrue: [decade := decade * 10].
	frequencyList := OrderedCollection new.
	decade do: [:freq | freq >= start ifTrue: [frequencyList add: freq]].
	[stop >= (decade last)] whileTrue: 
			[decade := decade * 10.
			decade do: [:freq | freq <= stop ifTrue: [frequencyList add: freq]]].
	^frequencyList! !

!LogScaleMorph methodsFor: 'drawing' stamp: 'hk 9/13/2007 20:31'!
drawMajorTicksOn: aCanvas 
	| scale x1 y1 y2 x yoffset randomLabel logTics startLog|
	"majorTics is for one decade, calc all tics in range"
	logTics := self calcMajorTics: majorTick .
	startLog := start log.
	scale := (self innerBounds width ) / (stop log - startLog) asFloat.
	yoffset := majorTickLength < 0 
		ifTrue: [ majorTickLength abs + 1]
		ifFalse: [1].
	caption ifNotNil: 
			[captionAbove 
				ifFalse: 
					[randomLabel := StringMorph contents: 'Foo'.
					yoffset := yoffset + randomLabel height + 2]].
	tickPrintBlock ifNotNil: 
			[labelsAbove 
				ifFalse: 
					[randomLabel := StringMorph contents: '50'.
					yoffset := yoffset + randomLabel height + 2]].
	x1 := self innerBounds left.
	y1 := self innerBounds bottom - yoffset.
	y2 := y1 - majorTickLength.
	logTics 
		do: 
			[:v | 
			x := x1 + (scale * (v log - startLog)).
			aCanvas 
				line: x @ y1
				to: x @ y2
				width: 1
				color: Color black.]! !

!LogScaleMorph methodsFor: 'drawing' stamp: 'hk 9/13/2007 20:30'!
drawMinorTicksOn: aCanvas 
	| scale x1 y1 y2 x yoffset randomLabel logTics startLog|
	logTics := self calcMinorTics .
	startLog := start log.
	scale := (self innerBounds width) / (stop log - startLog) asFloat.
	yoffset := majorTickLength < 0 
				ifTrue: [majorTickLength abs + 1]
				ifFalse: [1].
	caption ifNotNil: 
			[captionAbove 
				ifFalse: 
					[randomLabel := StringMorph contents: 'Foo'.
					yoffset := yoffset + randomLabel height + 2]].
	tickPrintBlock ifNotNil: 
			[labelsAbove 
				ifFalse: 
					[randomLabel := StringMorph contents: '50'.
					yoffset := yoffset + randomLabel height + 2]].
	x1 := self innerBounds left.
	y1 := self innerBounds bottom - yoffset.
	y2 := y1 - minorTickLength.
	logTics do: 
			[:v | 
			x := x1 + (scale * (v log - startLog)).
			aCanvas 
				line: x @ y1
				to: x @ y2
				width: 1
				color: Color black]! !


!LogScaleMorph methodsFor: 'geometry' stamp: 'hk 8/29/2007 12:48'!
checkExtent: newExtent 
	| pixPerTick newWidth |
	pixPerTick := newExtent x - (self borderWidth * 2) - 1 / ((stop log - start log) asFloat / minorTick).
	pixPerTick := pixPerTick
				detentBy: 0.1
				atMultiplesOf: 1.0
				snap: false.
	newWidth := pixPerTick * ((stop log - start log) asFloat / minorTick) + (self borderWidth * 2) + 1.
	^ (newWidth @ newExtent y).! !


!LogScaleMorph methodsFor: 'initialization' stamp: 'hk 8/29/2007 12:35'!
initialize
	"initialize the state of the receiver"
	super initialize.
	""
	
	start := 10.
	stop := 1000.
	minorTick := 10.
	majorTick := #(1 3).
	minorTickLength := 3.
	majorTickLength := 10.
	caption := nil.
	tickPrintBlock := [:v | v printString].
	labelsAbove := true.
	captionAbove := true! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

LogScaleMorph class
	instanceVariableNames: ''!

!LogScaleMorph class methodsFor: 'examples' stamp: 'hk 8/29/2007 13:55'!
example1
	"Example 1 captions and labels above, ticks point up"
	^ (self new
		start: 1 "0 is a bad start for logarithmic scales"
		stop: 150
		minorTick: 0.2
		minorTickLength: 2
		majorTick: #(1 2 3 5)
		majorTickLength: 10
		caption: 'Example 1'
		tickPrintBlock: [:v | v printString];
		width: 300) openInWorld! !

!LogScaleMorph class methodsFor: 'examples' stamp: 'hk 8/29/2007 13:57'!
example2
	"Example 2 captions and labels above, ticks point down"
	^ (self new
		start: 1 "0 is a bad start for logarithmic scales"
		stop: 150
		minorTick: 0.5
		minorTickLength: 2
		majorTick: #(1 3 5)
		majorTickLength: -10
		caption: 'Example 2'
		tickPrintBlock: [:v | v printString];
		width: 300) openInWorld! !

!LogScaleMorph class methodsFor: 'examples' stamp: 'hk 8/29/2007 14:02'!
example3
	"Example 3 caption above, labels below, ticks point down"
	^ (self new
		start: 0.1 "0 or negative is a bad start for logarithmic scales"
		stop: 47
		minorTick:0.5
		minorTickLength: 2
		majorTick: #(1 2 3 5)
		majorTickLength: -10
		caption: 'Example 3'
		tickPrintBlock: [:v | v asFloat printString]
		labelsAbove: false
		captionAbove: true;
		color: Color lightBlue;
		width: 300) openInWorld! !

!LogScaleMorph class methodsFor: 'examples' stamp: 'hk 8/29/2007 14:06'!
example4
	"Example 4 caption below, labels above, ticks point up"
	^ (self new
		start: 100000
		stop: 300000
		minorTick: 0.1
		minorTickLength: 2
		majorTick: #(1 2 3)
		majorTickLength: 10
		caption: 'Example 4'
		tickPrintBlock: [:v | v < 1000 
							ifTrue: [v printString, 'Hz'] 
							ifFalse:[(v / 1000) asFloat printString , 'kHz']]
		labelsAbove: true
		captionAbove: false;
		color: Color lightOrange;
		width: 300) openInWorld! !


Object subclass: #SignalPlotter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Dipole Radiator'!
!SignalPlotter commentStamp: 'hk 8/20/2009 20:59' prior: 0!
I can plot the contents of an array of  data in several ways.

Examples are on the class side.

I just draw on Display and development is driven by the kind of data I need to plot.
The basic idea is stolen from Dan Ingalls FFT, see FFT>plot:in: and usage in FFT>>test for rectangular plots.

What is a contour plot:
A polar plot in acoustics gives you the loudness for one frequency depending of the direction if the listener walks a circle around a loudspeaker. If you prefer stay seated and turn the loudspeaker :-)) One can draw this for two or three frequencies in one diagram but what if you want to know for a lot of frequencies?

A contour plot will draw the frequency on the x-axis, direction on x-axis and code the signal strength in color shades, e.g. red being loud and blue being quiet.
So you need an array of arrays of data, Each array contains the amplitudes for all directions for a particular frequency.!


!SignalPlotter methodsFor: 'accessing' stamp: 'hk 10/15/2008 20:42'!
unitCircle: noOfIncrsPerCircle
	"return a FloatArray where the first noOfincrs are the x values, then noOf incrs y values. FloatArray because of fast scaling
(SignalPlotter new unitCircle: 360) explore
"

	| arrX arrY angDiff |
	angDiff := Float pi * 2.0 / noOfIncrsPerCircle.
	arrX := (0.0 to: (Float pi * 2.0 - angDiff ) by: angDiff )
				collect: [: ang| ang cos].
	arrY := (0.0 to: (Float pi * 2.0 - angDiff ) by: angDiff )
				collect: [: ang| ang sin].
	^(arrX , arrY) asFloatArray! !


!SignalPlotter methodsFor: 'contour plots' stamp: 'hk 11/5/2008 11:11'!
contourPlot: aFrequencyAngleLevelCollection in: aRectangle lowValue: aFloat colors: anArray
	"only with an odd number of angles due to the type of the diagram"
	| noBands noDirections dx dy x y minValue|
	noBands := aFrequencyAngleLevelCollection size.
	noDirections := aFrequencyAngleLevelCollection first size.
	noDirections odd ifTrue: 
		[self inform: 'odd number of directions make lousy diagrams'].
	minValue := (aFrequencyAngleLevelCollection collect: [:each| each min ]) min. 
	dx := (aRectangle width / noBands).  "dx@dy is the size of a coloured rectangle"
	dy := (aRectangle height / (noDirections + 1)). "180° is plotted twice"
	y := aRectangle origin y + 2.
	x := aRectangle origin x.
	Display fillWhite: aRectangle; border: (aRectangle expandBy: 2) width: 2.
	aFrequencyAngleLevelCollection withIndexDo: "columns"
		[:dirArray :index|
		self fillColumnCenteredIn: aRectangle x: x dx: dx y: y  dy: dy values: dirArray colors: anArray lowValue: aFloat.
		 x := x + dx.
		].! !

!SignalPlotter methodsFor: 'contour plots' stamp: 'hk 11/5/2008 19:30'!
fillColumnCenteredIn: aRectangle x: xNumber dx: dxNumber y: yNumber dy: dyNumber values: anOrderedCollection colors: anArrayOfColors lowValue: aFloat 
	"Values describe a circle but a column displays 0 +- 180°"

	| rectOrigin rectCorner indexArray fillRule move |
	fillRule := Form over.
	move := 0.0 @ dyNumber.
	rectOrigin := xNumber @ (yNumber - (dyNumber / 2.0)).
	rectCorner := (xNumber + dxNumber) @ (yNumber + (dyNumber / 2.0)).
	indexArray := self makeIndexArray: anOrderedCollection size.
	indexArray do: 
			[:newIndex | 
			Display 
				fill: (rectOrigin corner: rectCorner)
				rule: fillRule
				fillColor: (self 
						getValueColor: (anOrderedCollection at: newIndex)
						from: anArrayOfColors
						lowValue: aFloat).
			rectCorner := rectCorner + move.
			rectOrigin := rectOrigin + move].! !

!SignalPlotter methodsFor: 'contour plots' stamp: 'hk 11/5/2008 09:33'!
getValueColor: aFloat from: anArrayOfColors lowValue: anotherFloat
	"get the corresponding colour for the value, values lower than lowValue get the color for the lowest value. Values are expected between 0db and -x db"
	|valueDiff index maxIndex|
	aFloat > 0.01 ifTrue: [^anArrayOfColors first].
	aFloat <= anotherFloat ifTrue: [^anArrayOfColors last].
	maxIndex := anArrayOfColors size.
	valueDiff := anotherFloat / maxIndex .
	index := (aFloat / valueDiff) truncated + 1.
	^anArrayOfColors at: index .! !

!SignalPlotter methodsFor: 'contour plots' stamp: 'hk 11/5/2008 10:12'!
makeIndexArray: numEntries 
	"Array of values comes from 0° to 360° we want -180 to 180, with 180 displayed twice"

	| indexArray index |
	indexArray := Array new: numEntries + 1.
	index := 1.
	numEntries / 2 + 1 to: 1 by: -1
		do: 
			[:newIndex | 
			indexArray at: index put: newIndex.
			index := index + 1].
	numEntries to: index copy by: - 1
		do: 
			[:newIndex | 
			indexArray at: index put: newIndex.
			index := index + 1].
	indexArray at: index put: numEntries / 2 + 1.
	^indexArray! !


!SignalPlotter methodsFor: 'polar plots' stamp: 'hk 10/15/2008 10:33'!
plotCenterCross: rect

	| pen dx dy center centerX centerY|
	pen := Pen new.  pen up.
	center := rect center.
	centerX := center x.
	centerY := center y.
	dx := rect width / 2.
	dy := rect height / 2.
	pen goto: (centerX - dx)@ centerY.
	pen down.
	pen goto: centerX + dx @ centerY.
	pen up.
	pen goto: centerX@(centerY - dy).
	pen down.
	pen goto: centerX @ (centerY + dy).
! !

!SignalPlotter methodsFor: 'polar plots' stamp: 'hk 10/15/2008 21:23'!
plotCircleRadius: aNumber unitCircle: aFloatArray center: aPoint
	"comment stating purpose of message"

	| circle pen indexOffset index1 |
	circle := (aFloatArray * aNumber) asArray.
	indexOffset := aFloatArray size / 2.
	index1 := 2.
	pen := Pen new up.
	pen goto: (circle at: 1) @ (circle at: 1 + indexOffset) + aPoint . 
	pen down.
	(index1 to: indexOffset -1) do:
		[:index| pen goto: ((circle at: index) @ (circle at: index + indexOffset)) +
							 aPoint ] .
	index1 := 2.
	pen goto: ((circle at: 1) @ (circle at: 1+ indexOffset)) + 	aPoint 
	! !

!SignalPlotter methodsFor: 'polar plots' stamp: 'hk 10/15/2008 22:07'!
plotPolar: samples into: rect
	|scale offset max values angDiff pen radius index|
	max := samples max.
	scale := (rect height min: rect width) / 2.0 / max.
	offset := rect center.
	values := samples * scale.
	angDiff := Float pi * 2.0 / values size.
	pen := Pen new up.
	pen color: Color red.
	radius := values first.
	pen goto: (0 @ (radius * -1) ) + offset.
	pen down.
	index := 1.
	(0.0 to: Float pi * 2.0 - angDiff  by: angDiff) do:
		[: ang| radius := values at: index.
		pen goto: (ang sin @ (ang cos * -1.0)) * radius + offset.
		index := index + 1.
		"(Delay forMilliseconds: 1) wait"].
	radius  := values first.
	pen goto: (0  @ (radius * -1) ) + offset.
	
	! !

!SignalPlotter methodsFor: 'polar plots' stamp: 'hk 10/16/2008 08:33'!
plotPolar: samples into: rect scale: aFloat
	self plotPolar: samples  into: rect  scale: aFloat  color: Color red
	
	! !

!SignalPlotter methodsFor: 'polar plots' stamp: 'hk 10/16/2008 08:31'!
plotPolar: samples into: rect scale: aFloat color: aColor
	|offset values angDiff pen radius index|
	offset := rect center.
	values := samples * aFloat.
	angDiff := Float pi * 2.0 / values size.
	pen := Pen new up.
	pen color: aColor.
	radius := values first.
	pen goto: (0 @ (radius * -1) ) + offset.
	pen down.
	index := 1.
	(0.0 to: Float pi * 2.0 - angDiff  by: angDiff) do:
		[: ang| radius := values at: index.
		pen goto: (ang sin @ (ang cos * -1.0)) * radius + offset.
		index := index + 1.
		"(Delay forMilliseconds: 1) wait"].
	radius  := values first.
	pen goto: (0  @ (radius * -1) ) + offset.
	
	! !

!SignalPlotter methodsFor: 'polar plots' stamp: 'hk 10/16/2008 15:55'!
polarPlot: samples in: rect 
	"Throw-away code :-)) in FFT stolen to: just to check out a couple of examples
	now changed to Polar, assuming samples is radii and 360/samples size is the turning increment in degrees"

	| max radius unitCircle radDiff scale|
	Display
		fillWhite: rect;
		border: (rect expandBy: 2) width: 2.
	
	max := samples max.
	scale := (rect height min: rect width) / 2.0 / max.
	max printString displayOn: Display at: (rect center x + 2) @ (rect top).
	(max * 0.75) printString displayOn: 
			Display at: (rect center x + 2) @ 
							(rect top + (rect bottom * 0.125) ).
	(max * 0.5) printString displayOn: 
			Display at: (rect center x + 2) @ 
							(rect top + (rect bottom * 0.25) ).
	(max * 0.25) printString displayOn: 
			Display at: (rect center x + 2) @ 
							(rect top + (rect bottom * 0.375) ).
	radius := (rect height min: rect width) / 2.
	radDiff := radius / 4.
	self plotCenterCross: rect.
	unitCircle := self unitCircle: 360.
	4 timesRepeat: 
			[self 
				plotCircleRadius: radius
				unitCircle: unitCircle
				center: rect center.
			radius := radius - radDiff].
	self plotPolar: samples into: rect scale: scale . ! !


!SignalPlotter methodsFor: 'rectangular plots' stamp: 'hk 7/21/2007 07:25'!
plot: samples in: rect
	"Throw-away code :-)) in FFT stolen to: just to check out a couple of examples"
	| min max x dx pen y |
	Display fillWhite: rect; border: (rect expandBy: 2) width: 2.
	min := 1.0e30.  max := -1.0e30.
	samples do:
		[:v |
		min := min min: v.
		max := max max: v].
	pen := Pen new.  pen up.
	x := rect left.
	dx := rect width asFloat / samples size.
	samples do:
		[:v |
		y := (max-v) / (max-min) * rect height asFloat.
		pen goto: x asInteger @ (rect top + y asInteger).
		pen down.
		x := x + dx].
	max printString displayOn: Display at: (x+2) @ (rect top-9).
	min printString displayOn: Display at: (x+2) @ (rect bottom - 9)! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

SignalPlotter class
	instanceVariableNames: ''!

!SignalPlotter class methodsFor: 'samples polar' stamp: 'hk 8/20/2009 21:23'!
sampleContourPlot
	" Below the contour plot the on axis signal strength is plotted.
	This shows the interference of two speakers back to back, the back one driven with inverse phase.
	SignalPlotter sampleContourPlot
	"
|radiator plotter samples colors rect sc loFreq hiFreq| 
loFreq := 20.0.
hiFreq := 600.0.
radiator := DipoleRadiator new .
radiator listeningDistance: 4.0; displacement: 1.0; turningCenterDisplacement: 0.0.
radiator rearPhase: 0.0 ; rearDelaySelector: #rearDelay.
plotter := SignalPlotter new.
rect := (5@5 extent: 500@292).
samples := radiator amplitudeDistributionLoFreq: loFreq hiFreq: hiFreq noBands: 50 noDirections: 72 log: true.
"minVal := (samples collect: [:each | each min]) min."
"colors := (Color green darkShades: 10) reversed."
colors := (Color red mix: Color blue shades: 30) .
plotter contourPlot: samples in: rect lowValue: -30.0 colors: colors.
(LogScaleMorph new
		start: loFreq 
		stop: hiFreq 
		minorTick: 1
		minorTickLength: 5
		majorTick: #(1  2 3 5)
		majorTickLength: -10
		caption: 'Frequency'
		tickPrintBlock: [:v | v < 1000 ifTrue: [v printString, 'Hz'] ifFalse: [(v / 1000) printString, 'kHz']]
		labelsAbove: false
		captionAbove: false;
		width: 500;
		color: Color white;
		position: 5@300) openInWorld.
sc := ScaleMorph new
	start: -180
	stop: 180
	minorTick: 10
	minorTickLength: 5
	majorTick: 60
	majorTickLength: 10
	caption: 'Degrees'
	tickPrintBlock: [:v| v printString];
	color: Color white;
	width: 292.
sc := TransformationMorph new asFlexOf: sc.
sc angle: Float pi / -2.0;
	position: 507@5.
sc   openInWorld .
sc := nil.
samples := radiator spectrumAtListenerPositionLevel: 1000000.0 loFreq: loFreq hiFreq: hiFreq .
samples withIndexDo: [:sample :index| samples at: index put: sample log * 10.0].
plotter plot: samples in: (5@355 extent: 500@200).

! !

!SignalPlotter class methodsFor: 'samples polar' stamp: 'hk 8/20/2009 20:23'!
samplePolarPlot
	"
	Display restoreAfter: [ SignalPlotter samplePolarPlot]
	"
	|data1 data2 rect|
	data1 := FloatArray new: 180 withAll: 3.0.
	data2 := FloatArray new: 180 withAll: 7.0.
	rect := 5@5 extent: 300@300.
	self new polarPlot: data1, data2 in: rect.
	! !
